/*
 * This File is part of Davix, The IO library for HTTP based protocols
 * Copyright (C) CERN 2016
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifndef DAVIX_FEATURES
#define DAVIX_FEATURES

#define DAVIX_MAJOR 0
#define DAVIX_MINOR 8
#define DAVIX_PATCH 6
#define DAVIX_MINIPATCH dirty
#define DAVIX_VERSION 0.8.6.dirty
#define DAVIX_VERSION_STRING "0.8.6.dirty"

#define DAVIX_API_VERSION 20160621

#define DAVIX_HAS_AWS_V4     1
#define DAVIX_HAS_AWS_TOKENS 1

#endif
